/**
* @file Scripts to convert SEDRA 3 text database to JavaScript
* @version 1.2.9
* @author Greg Borota
* @copyright (c) 2017 Greg Borota.
*
* Based on SEDRA 3, copyright (c) 1996 GEORGE A. KIRAZ
*
* @license MIT
*
* The following license applies to all parts of this software except as
* documented below:
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* Files located in the node_modules and vendor directories are externally
* maintained libraries used by this software, which have their own licenses;
* their terms may differ from the terms above.
*
* Sedrajs contains a JavaScript representation of SEDRA 3 text database, with
* additional supporting JS data structures and utilities. SEDRA database portion
* is subject to the restrictions imposed by the authors of SEDRA 3 text database.
* The database was originally downloaded from <http://www.bethmardutho.org/>.
*
*       For additional information on the structure of the Database, see
*          G. Kiraz, `Automatic Concordance Generation of Syriac Texts',
*          in VI Symposium Syriacum 1992, ed. R. Lavenant, Orientalia
*          Christiana Analecta 247, Rome, 1994.
*
*       You are allowed to use SEDRA III for personal and academic
*       purposes provided that:
*       1. You do NOT redistribute any altered versions of the files.
*       2. You do NOT redistribute any files for any kind of profit.
*       3. You acknowledge in any publication whose results make use of
*          SEDRA III, by any means, using a formula similar to the
*          following:
*       "This work makes use of the Syriac Electronic Data
*        Retrieval Archive (SEDRA) by George A. Kiraz, distributed
*        by the Syriac Computing Institute."
*       You should also cite the paper mentioned above.
*
* See sedra/SEDRA3.DOC for additional details.*/

// https://peshitta.github.io
// https://sedra.bethmardutho.org/about/fonts
// http://cal1.cn.huc.edu/searching/fullbrowser.html

import { join } from 'path';
import { mkdir, readFile, realpath, writeFile } from 'fs';
import { getEnglish, getEtymology, getLexemes, getRoots, getUbs, getWords } from 'sedra-parse';

/** @module convert */
var realpathAsync = function (path$$1) { return new Promise(function (resolve, reject) {
    realpath(path$$1, 'utf8', function (err, resolvedPath) {
      if (err) {
        reject(err);
      } else {
        resolve(resolvedPath);
      }
    });
  }); };

var mkdirAsync = function (path$$1) { return new Promise(function (resolve, reject) {
    mkdir(path$$1, function (err) {
      if (err && err.code !== 'EEXIST') {
        reject(err);
      } else {
        resolve();
      }
    });
  }); };

var readFileAsync = function (path$$1) { return new Promise(function (resolve, reject) {
    readFile(path$$1, 'utf8', function (err, data) {
      if (err) {
        reject(err);
      } else {
        resolve(data);
      }
    });
  }); };

var writeFileAsync = function (file, data) { return new Promise(function (resolve, reject) {
    writeFile(file, data, 'utf8', function (err) {
      if (err) {
        reject(err);
      } else {
        resolve();
      }
    });
  }); };

var outDir = __dirname + "/../";
var outSedraDir = outDir + "sedra/";
var throwError = function (error) {
  throw error;
};

/**
 * Sedra db content converter: callback to transform content read from disk.
 * @static
 * @callback converter
 * @param { string } content Sedra db text file content
 * @returns { string|object } Converted content
 */

/**
 * Read Sedra db file asynchronously and returns converted content promise
 * @static
 * @param { string } db Sedra text database file name
 * @param { converter } converter Content converter
 * @returns { Promise.<string|object> } Converted content promise
 */
var readDb = function (db, converter) { return realpathAsync(join(__dirname, '../../sedra', db))
    .then(function (file) { return readFileAsync(file)
        .then(function (content) { return converter(content); })
        .catch(throwError); }
    )
    .catch(throwError); };

/**
 * Write content asynchronously and return promise
 * @static
 * @param { string } filePath File path to write to
 * @param { converter } content File content to save
 * @returns { Promise } File write promise
 */
var writeDb = function (filePath, content) { return writeFileAsync(filePath, content)
    .then(function () {
      realpathAsync(filePath)
        .then(function (file) {
          global.console.log(("Saved '" + file + "'"));
        })
        .catch(throwError);
    })
    .catch(throwError); };

/**
 * Read all of Sedra 3 text database and convert it to JavaScript
 * @static
 * @const
 * @returns { Promise } Promise for JavaScript database
 */
var convertDb = function () {
  var rootJs = null;
  var lexemeJs = null;
  var wordJs = null;
  var englishJs = null;
  var etymologyJs = null;
  var ubsJs = null;
  var rootPromise = readDb('ROOTS.TXT', getRoots).then(function (js) {
    rootJs = js;
  });
  var lexemePromise = readDb('LEXEMES.TXT', getLexemes).then(function (js) {
    lexemeJs = js;
  });
  var wordPromise = readDb('WORDS.TXT', getWords).then(function (js) {
    wordJs = js;
  });
  var englishPromise = readDb('ENGLISH.TXT', getEnglish).then(function (js) {
    englishJs = js;
  });
  var etymologyPromise = readDb('ETIMOLGY.TXT', getEtymology).then(function (js) {
    etymologyJs = js;
  });
  var ubsPromise = readDb('BFBS.TXT', getUbs).then(function (js) {
    ubsJs = js;
  });

  return Promise.all([
    rootPromise,
    lexemePromise,
    wordPromise,
    englishPromise,
    etymologyPromise,
    ubsPromise,
    mkdirAsync(outSedraDir)
  ])
    .then(function () {
      var rootContent = "o=" + rootJs;
      var lexemeContent = "m=" + lexemeJs;
      var wordContent = "d=" + (wordJs.words);
      var englishContent = "n=" + (englishJs.english);
      var etymologyContent = "y=" + (etymologyJs.etymology);
      var ubsContent = "u=" + (ubsJs.ubs);
      var content = "" + rootContent + lexemeContent + wordContent + englishContent + etymologyContent + ubsContent;

      var rootImport = 'makeRoot as r';
      var lexemeImport = 'makeLexeme as l';
      var wordImport = 'makeWord as w';
      var englishImport = 'makeEnglish as e';
      var etymologyImport = 'makeEtymology as t';
      var importText = 'import{';
      var fromText = "}from'sedra-model'";
      var o = 'o';
      var m = 'm';
      var d = 'd';
      var n = 'n';
      var y = 'y';
      var u = 'u';
      var rootExport = o + " as roots";
      var lexemeExport = m + " as lexemes";
      var wordExport = d + " as words";
      var englishExport = n + " as english";
      var etymologyExport = y + " as etymology";
      var ubsExport = u + " as ubs";
      var exportText = 'export{';

      var rootHeader = "" + importText + rootImport + fromText + ";export default ";
      var lexemeHeader = "" + importText + lexemeImport + fromText + ";export default ";
      var wordHeader = "" + importText + wordImport + fromText + ";export default ";
      var englishHeader = "" + importText + englishImport + fromText + ";export default ";
      var etymologyHeader = "" + importText + etymologyImport + fromText + ";export default ";
      var defaultExport = 'export default ';
      var moduleHeader = "" + importText + rootImport + "," + lexemeImport + "," + wordImport + "," + englishImport + "," + etymologyImport + fromText + ";var " + o + "," + m + "," + d + "," + n + "," + y + "," + u + ";";

      var moduleFooter = "" + exportText + rootExport + "," + lexemeExport + "," + wordExport + "," + englishExport + "," + etymologyExport + "," + ubsExport + "};";
      var umdHeader =
        "!function(g,f){'object'==typeof exports&&'undefined'!=typeof module?f(exports,require('sedra-model')):'function'==typeof define&&define.amd?define(['exports','sedra-model'],f):f(g.sedrajs={},g.sedraModel)}(this,function(x,s){'use strict';var r=s.makeRoot,l=s.makeLexeme,w=s.makeWord,e=s.makeEnglish,t=s.makeEtymology,o,m,d,n,y,u;";
      var umdFooter =
        "x.roots=o,x.lexemes=m,x.words=d,x.english=n,x.etymology=y,x.ubs=u,Object.defineProperty(x,'__esModule',{value:!0})});";

      var rootFile = outSedraDir + "roots.js";
      var lexemeFile = outSedraDir + "lexemes.js";
      var wordFile = outSedraDir + "words.js";
      var englishFile = outSedraDir + "english.js";
      var etymologyFile = outSedraDir + "etymology.js";
      var ubsFile = outSedraDir + "ubs.js";
      var noWFile = outSedraDir + "noW.js";
      var noYFile = outSedraDir + "noY.js";
      var englishLidFile = outSedraDir + "englishLid.js";
      var etymologyLidFile = outSedraDir + "etymologyLid.js";
      var referenceFile = outSedraDir + "ubsReference.js";

      var moduleFile = outDir + "sedrajs.esm.js";
      var umdFile = outDir + "sedrajs.js";

      return Promise.all([
        writeDb(rootFile, ("" + rootHeader + rootJs)),
        writeDb(lexemeFile, ("" + lexemeHeader + lexemeJs)),
        writeDb(wordFile, ("" + wordHeader + (wordJs.words))),
        writeDb(englishFile, ("" + englishHeader + (englishJs.english))),
        writeDb(etymologyFile, ("" + etymologyHeader + (etymologyJs.etymology))),
        writeDb(ubsFile, ("" + defaultExport + (ubsJs.ubs))),
        writeDb(noWFile, ("" + defaultExport + (wordJs.noW))),
        writeDb(noYFile, ("" + defaultExport + (wordJs.noY))),
        writeDb(englishLidFile, ("" + defaultExport + (englishJs.lids))),
        writeDb(etymologyLidFile, ("" + defaultExport + (etymologyJs.lids))),
        writeDb(referenceFile, ("" + defaultExport + (ubsJs.reference))),

        writeDb(moduleFile, ("" + moduleHeader + content + moduleFooter)),
        writeDb(umdFile, ("" + umdHeader + content + umdFooter))
      ]);
    })
    .catch(throwError);
};

export { readDb, writeDb, convertDb, mkdirAsync as mkDir };
